////////////////////////////////////////////////////////////
//  Puts atoms randomly in a box, subject to distance     //
//  cutoffs                                               //
//                                                        //
//  JKC, October 2009                                     //
////////////////////////////////////////////////////////////

#include <iostream>
#include <fstream>
#include <cmath>
#include <stdlib.h>

using namespace std;

bool SHELLMODEL = true;

double randone()
{
    // returns a number uniformly distributed between 0 and 1
    double rr = rand();
    return rr/static_cast<double>(RAND_MAX);
}

int rounding (double x)
{
    // rounds a number to the nearest integer
    int intx = static_cast<int>(x); // truncates
    double dx = x - intx;
    if(dx<0.5){return intx;}
    else{return (intx+1);}
}

int main(int argc, char **argv)
{
    srand(time(NULL));

    if(argc<13)
    {
	cerr << "Usage: " << argv[0] << " <Si-O cutoff (A)> <M-O cutoff (A)> <X-X cutoff (A)> <natoms> <density (g/cm3)> <density correction (can be 0.95)> <SiO2 mol %> <Na2O mol %> <CaO mol %> <CuO mol %> <P2O5 mol %> <Cu2O mol %>" << endl;
	return 1;
    }

    int natoms_intended = atoi(argv[4]);

    double mol_SiO2 = atof(argv[7]);
    double mol_Na2O = atof(argv[8]);
    double mol_CaO = atof(argv[9]);
    double mol_CuO = atof(argv[10]);
    double mol_P2O5 = atof(argv[11]);
    double mol_Cu2O = atof(argv[12]);

    // sanity check on percentages
    if(fabs(mol_SiO2+mol_Na2O+mol_CaO+mol_CuO+mol_P2O5+mol_Cu2O-100.0)>0.03)
    {
	cerr << "Percentages do not add up to 100.0! " << mol_SiO2+mol_Na2O+mol_CaO+mol_CuO+mol_P2O5+mol_Cu2O << endl;
	return 1;
    }

    // compute the total number of formula units

    int nunits = rounding(100.0*natoms_intended/(mol_SiO2*3.0 + mol_Na2O*3.0 + mol_CaO*2.0 + mol_CuO*2.0 + mol_P2O5*7.0 + mol_Cu2O*3.0));
    
    // compute the number of formula units of each molecule

    int nunits_SiO2 = rounding(mol_SiO2*nunits/100.0);
    int nunits_Na2O = rounding(mol_Na2O*nunits/100.0);
    int nunits_CaO = rounding(mol_CaO*nunits/100.0);
    int nunits_CuO = rounding(mol_CuO*nunits/100.0);
    int nunits_P2O5 = rounding(mol_P2O5*nunits/100.0);
    int nunits_Cu2O = rounding(mol_Cu2O*nunits/100.0);


    // numbers of atoms

    int nO = 2*nunits_SiO2 + nunits_Na2O + nunits_CaO + nunits_CuO + 5*nunits_P2O5+ nunits_Cu2O;
    int nSi = nunits_SiO2;
    int nNa = 2*nunits_Na2O;
    int nCa = nunits_CaO;
    int nCu = nunits_CuO;
    int nP = 2*nunits_P2O5;
    int nCu2 = 2*nunits_Cu2O;
    int natoms = nO+nSi+nNa+nCa+nCu+nP+nCu2;

    // output true composition

    int nunits_true = nunits_SiO2+nunits_Na2O+nunits_CaO+nunits_CuO+nunits_P2O5+nunits_Cu2O;

    cerr << "Simulating " << natoms << " atoms" << endl;
    cerr << 100.0*double(nunits_SiO2)/(double)nunits_true << " mol% SiO2" << endl;
    cerr << 100.0*double(nunits_Na2O)/(double)nunits_true << " mol% Na2O" << endl;
    cerr << 100.0*double(nunits_CaO)/(double)nunits_true << " mol% CaO" << endl;
    cerr << 100.0*double(nunits_CuO)/(double)nunits_true << " mol% CuO" << endl;
    cerr << 100.0*double(nunits_P2O5)/(double)nunits_true << " mol% P2O5" << endl;
    cerr << 100.0*double(nunits_Cu2O)/(double)nunits_true << " mol% Cu2O" << endl;
     
    // define mass
    double mass = (nO*15.9994+ nSi*28.0855 + nNa*22.989769 + nCa*40.078 + nCu*63.545 + nP*30.973762 + nCu2*63.546)*1.66053886e-24; // in grams

    cerr << "mass: " << mass << endl;

    // required density
    double density = atof(argv[5])*atof(argv[6]); // g/cm^3

    // hence the volume
    double volume = mass/density; // in cm^3

    // define the box size
    double box = pow(volume,1.0/3.0)*1e8; // angstroms
    
    cerr << box << endl;

    // put the oxygen atoms in

    double cutoff_OO = atof(argv[3])/box;

    double Opos[nO][3]; // oxygen atom positions
    int ACCEPT;
    double trialpos[3],dx,dy,dz,dr2;
    int accepted=0;
    do{
	ACCEPT=1;
	// choose trial position
	trialpos[0] = randone();
	trialpos[1] = randone();
	trialpos[2] = randone();

	// check interatomic distances
	for(int j=0; j<accepted && ACCEPT==1; j++)
	{
	    dx = Opos[j][0] - trialpos[0];
	    dy = Opos[j][1] - trialpos[1];
	    dz = Opos[j][2] - trialpos[2];
	    
	    // correct for PBCs
	    while(dx>0.5){dx-=1.0;}
	    while(dy>0.5){dy-=1.0;}
	    while(dz>0.5){dz-=1.0;}
	    while(dx<-0.5){dx+=1.0;}
	    while(dy<-0.5){dy+=1.0;}
	    while(dz<-0.5){dz+=1.0;}
	
	    dr2 = dx*dx + dy*dy + dz*dz;
	    if(dr2<cutoff_OO*cutoff_OO){ACCEPT=0;}
	}
	if(ACCEPT==1)
	{
	    // then accept the atom position
	    Opos[accepted][0] = trialpos[0];
	    Opos[accepted][1] = trialpos[1];
	    Opos[accepted][2] = trialpos[2];
	    accepted++;
            if(accepted%100==0){cerr << accepted << " ";}
	}

	
    }while(ACCEPT==0 || (ACCEPT==1 && accepted<nO)); 
    
    // put the silicon atoms in 

    double cutoff_SiO = atof(argv[1])/box;
    double cutoff_SiSi = atof(argv[3])/box;

    double Sipos[nSi][3]; // silicon atom positions
    accepted=0;
   
    do{
	ACCEPT=1;
	// choose trial position
	trialpos[0] = randone();
	trialpos[1] = randone();
	trialpos[2] = randone();
	
	// check Si-O interatomic distances
	for(int j=0; j<nO && ACCEPT==1; j++)
	{
	    dx = Opos[j][0] - trialpos[0];
	    dy = Opos[j][1] - trialpos[1];
	    dz = Opos[j][2] - trialpos[2];
	    
	    // correct for PBCs
	    while(dx>0.5){dx-=1.0;}
	    while(dy>0.5){dy-=1.0;}
	    while(dz>0.5){dz-=1.0;}
	    while(dx<-0.5){dx+=1.0;}
	    while(dy<-0.5){dy+=1.0;}
	    while(dz<-0.5){dz+=1.0;}
	
	    dr2 = dx*dx + dy*dy + dz*dz;
	    if(dr2<cutoff_SiO*cutoff_SiO){ACCEPT=0;}
	}

	// check Si-Si interatomic distances
	for(int j=0; j<accepted && ACCEPT==1; j++)
	{
	    dx = Sipos[j][0] - trialpos[0];
	    dy = Sipos[j][1] - trialpos[1];
	    dz = Sipos[j][2] - trialpos[2];
	    
	    // correct for PBCs
	    while(dx>0.5){dx-=1.0;}
	    while(dy>0.5){dy-=1.0;}
	    while(dz>0.5){dz-=1.0;}
	    while(dx<-0.5){dx+=1.0;}
	    while(dy<-0.5){dy+=1.0;}
	    while(dz<-0.5){dz+=1.0;}
	
	    dr2 = dx*dx + dy*dy + dz*dz;
	    if(dr2<cutoff_SiSi*cutoff_SiSi){ACCEPT=0;}
	}

	if(ACCEPT==1)
	{
	    // then accept the atom position
	    Sipos[accepted][0] = trialpos[0];
	    Sipos[accepted][1] = trialpos[1];
	    Sipos[accepted][2] = trialpos[2];
	    accepted++;
	}
	
    }while(ACCEPT==0 || (ACCEPT==1 && accepted<nSi)); 


    // put sodium atoms in

    double cutoff_NaO = atof(argv[2])/box;
    double cutoff_NaSi = atof(argv[3])/box;
    double cutoff_NaNa = atof(argv[3])/box;

    double Napos[nNa][3]; // sodium atom positions
    accepted=0;
   
    do{
	ACCEPT=1;
	// choose trial position
	trialpos[0] = randone();
	trialpos[1] = randone();
	trialpos[2] = randone();
	
	// check Na-O interatomic distances
	for(int j=0; j<nO && ACCEPT==1; j++)
	{
	    dx = Opos[j][0] - trialpos[0];
	    dy = Opos[j][1] - trialpos[1];
	    dz = Opos[j][2] - trialpos[2];
	    
	    // correct for PBCs
	    while(dx>0.5){dx-=1.0;}
	    while(dy>0.5){dy-=1.0;}
	    while(dz>0.5){dz-=1.0;}
	    while(dx<-0.5){dx+=1.0;}
	    while(dy<-0.5){dy+=1.0;}
	    while(dz<-0.5){dz+=1.0;}
	
	    dr2 = dx*dx + dy*dy + dz*dz;
	    if(dr2<cutoff_NaO*cutoff_NaO){ACCEPT=0;}
	}

	// check Na-Si interatomic distances
	for(int j=0; j<nSi && ACCEPT==1; j++)
	{
	    dx = Sipos[j][0] - trialpos[0];
	    dy = Sipos[j][1] - trialpos[1];
	    dz = Sipos[j][2] - trialpos[2];
	    
	    // correct for PBCs
	    while(dx>0.5){dx-=1.0;}
	    while(dy>0.5){dy-=1.0;}
	    while(dz>0.5){dz-=1.0;}
	    while(dx<-0.5){dx+=1.0;}
	    while(dy<-0.5){dy+=1.0;}
	    while(dz<-0.5){dz+=1.0;}
	
	    dr2 = dx*dx + dy*dy + dz*dz;
	    if(dr2<cutoff_NaSi*cutoff_NaSi){ACCEPT=0;}
	}

	// check Na-Na interatomic distances
	for(int j=0; j<accepted && ACCEPT==1; j++)
	{
	    dx = Napos[j][0] - trialpos[0];
	    dy = Napos[j][1] - trialpos[1];
	    dz = Napos[j][2] - trialpos[2];
	    
	    // correct for PBCs
	    while(dx>0.5){dx-=1.0;}
	    while(dy>0.5){dy-=1.0;}
	    while(dz>0.5){dz-=1.0;}
	    while(dx<-0.5){dx+=1.0;}
	    while(dy<-0.5){dy+=1.0;}
	    while(dz<-0.5){dz+=1.0;}
	
	    dr2 = dx*dx + dy*dy + dz*dz;
	    if(dr2<cutoff_NaNa*cutoff_NaNa){ACCEPT=0;}
	}

	if(ACCEPT==1)
	{
	    // then accept the atom position
	    Napos[accepted][0] = trialpos[0];
	    Napos[accepted][1] = trialpos[1];
	    Napos[accepted][2] = trialpos[2];
	    accepted++;
	}
	
    }while(ACCEPT==0 || (ACCEPT==1 && accepted<nNa)); 


    // put the calcium atoms in

    double cutoff_CaO = atof(argv[2])/box;
    double cutoff_CaSi = atof(argv[3])/box;
    double cutoff_CaNa = atof(argv[3])/box;
    double cutoff_CaCa = atof(argv[3])/box;

    double Capos[nCa][3]; // calcium atom positions
    accepted=0;
    
    do{
	ACCEPT=1;
	// choose trial position
	trialpos[0] = randone();
	trialpos[1] = randone();
	trialpos[2] = randone();
	
	// check Ca-O interatomic distances
	for(int j=0; j<nO && ACCEPT==1; j++)
	{
	    dx = Opos[j][0] - trialpos[0];
	    dy = Opos[j][1] - trialpos[1];
	    dz = Opos[j][2] - trialpos[2];
	    
	    // correct for PBCs
	    while(dx>0.5){dx-=1.0;}
	    while(dy>0.5){dy-=1.0;}
	    while(dz>0.5){dz-=1.0;}
	    while(dx<-0.5){dx+=1.0;}
	    while(dy<-0.5){dy+=1.0;}
	    while(dz<-0.5){dz+=1.0;}
	
	    dr2 = dx*dx + dy*dy + dz*dz;
	    if(dr2<cutoff_CaO*cutoff_CaO){ACCEPT=0;}
	}

	// check Ca-Si interatomic distances
	for(int j=0; j<nSi && ACCEPT==1; j++)
	{
	    dx = Sipos[j][0] - trialpos[0];
	    dy = Sipos[j][1] - trialpos[1];
	    dz = Sipos[j][2] - trialpos[2];
	    
	    // correct for PBCs
	    while(dx>0.5){dx-=1.0;}
	    while(dy>0.5){dy-=1.0;}
	    while(dz>0.5){dz-=1.0;}
	    while(dx<-0.5){dx+=1.0;}
	    while(dy<-0.5){dy+=1.0;}
	    while(dz<-0.5){dz+=1.0;}
	
	    dr2 = dx*dx + dy*dy + dz*dz;
	    if(dr2<cutoff_CaSi*cutoff_CaSi){ACCEPT=0;}
	}

	// check Ca-Na interatomic distances
	for(int j=0; j<nNa && ACCEPT==1; j++)
	{
	    dx = Napos[j][0] - trialpos[0];
	    dy = Napos[j][1] - trialpos[1];
	    dz = Napos[j][2] - trialpos[2];
	    
	    // correct for PBCs
	    while(dx>0.5){dx-=1.0;}
	    while(dy>0.5){dy-=1.0;}
	    while(dz>0.5){dz-=1.0;}
	    while(dx<-0.5){dx+=1.0;}
	    while(dy<-0.5){dy+=1.0;}
	    while(dz<-0.5){dz+=1.0;}
	
	    dr2 = dx*dx + dy*dy + dz*dz;
	    if(dr2<cutoff_CaNa*cutoff_CaNa){ACCEPT=0;}
	}

	// check Ca-Ca interatomic distances
	for(int j=0; j<accepted && ACCEPT==1; j++)
	{
	    dx = Capos[j][0] - trialpos[0];
	    dy = Capos[j][1] - trialpos[1];
	    dz = Capos[j][2] - trialpos[2];
	    
	    // correct for PBCs
	    while(dx>0.5){dx-=1.0;}
	    while(dy>0.5){dy-=1.0;}
	    while(dz>0.5){dz-=1.0;}
	    while(dx<-0.5){dx+=1.0;}
	    while(dy<-0.5){dy+=1.0;}
	    while(dz<-0.5){dz+=1.0;}
	
	    dr2 = dx*dx + dy*dy + dz*dz;
	    if(dr2<cutoff_CaCa*cutoff_CaCa){ACCEPT=0;}
	}

	if(ACCEPT==1)
	{
	    // then accept the atom position
	    Capos[accepted][0] = trialpos[0];
	    Capos[accepted][1] = trialpos[1];
	    Capos[accepted][2] = trialpos[2];
	    accepted++;
	}
	
    }while(ACCEPT==0 || (ACCEPT==1 && accepted<nCa)); 
 
    // put the copper atoms in

    double cutoff_CuO = atof(argv[2])/box;
    double cutoff_CuSi = atof(argv[3])/box;
    double cutoff_CuNa = atof(argv[3])/box;
    double cutoff_CuCa = atof(argv[3])/box;
    double cutoff_CuCu = atof(argv[3])/box;

    double Cupos[nCu][3]; // calcium atom positions
    accepted=0;
    
    do{
	ACCEPT=1;
	// choose trial position
	trialpos[0] = randone();
	trialpos[1] = randone();
	trialpos[2] = randone();
	
	// check Cu-O interatomic distances
	for(int j=0; j<nO && ACCEPT==1; j++)
	{
	    dx = Opos[j][0] - trialpos[0];
	    dy = Opos[j][1] - trialpos[1];
	    dz = Opos[j][2] - trialpos[2];
	    
	    // correct for PBCs
	    while(dx>0.5){dx-=1.0;}
	    while(dy>0.5){dy-=1.0;}
	    while(dz>0.5){dz-=1.0;}
	    while(dx<-0.5){dx+=1.0;}
	    while(dy<-0.5){dy+=1.0;}
	    while(dz<-0.5){dz+=1.0;}
	
	    dr2 = dx*dx + dy*dy + dz*dz;
	    if(dr2<cutoff_CuO*cutoff_CuO){ACCEPT=0;}
	}

	// check Cu-Si interatomic distances
	for(int j=0; j<nSi && ACCEPT==1; j++)
	{
	    dx = Sipos[j][0] - trialpos[0];
	    dy = Sipos[j][1] - trialpos[1];
	    dz = Sipos[j][2] - trialpos[2];
	    
	    // correct for PBCs
	    while(dx>0.5){dx-=1.0;}
	    while(dy>0.5){dy-=1.0;}
	    while(dz>0.5){dz-=1.0;}
	    while(dx<-0.5){dx+=1.0;}
	    while(dy<-0.5){dy+=1.0;}
	    while(dz<-0.5){dz+=1.0;}
	
	    dr2 = dx*dx + dy*dy + dz*dz;
	    if(dr2<cutoff_CuSi*cutoff_CuSi){ACCEPT=0;}
	}

	// check Cu-Na interatomic distances
	for(int j=0; j<nNa && ACCEPT==1; j++)
	{
	    dx = Napos[j][0] - trialpos[0];
	    dy = Napos[j][1] - trialpos[1];
	    dz = Napos[j][2] - trialpos[2];
	    
	    // correct for PBCs
	    while(dx>0.5){dx-=1.0;}
	    while(dy>0.5){dy-=1.0;}
	    while(dz>0.5){dz-=1.0;}
	    while(dx<-0.5){dx+=1.0;}
	    while(dy<-0.5){dy+=1.0;}
	    while(dz<-0.5){dz+=1.0;}
	
	    dr2 = dx*dx + dy*dy + dz*dz;
	    if(dr2<cutoff_CuNa*cutoff_CuNa){ACCEPT=0;}
	}

	// check Cu-Ca interatomic distances
	for(int j=0; j<nCa && ACCEPT==1; j++)
	{
	    dx = Capos[j][0] - trialpos[0];
	    dy = Capos[j][1] - trialpos[1];
	    dz = Capos[j][2] - trialpos[2];
	    
	    // correct for PBCs
	    while(dx>0.5){dx-=1.0;}
	    while(dy>0.5){dy-=1.0;}
	    while(dz>0.5){dz-=1.0;}
	    while(dx<-0.5){dx+=1.0;}
	    while(dy<-0.5){dy+=1.0;}
	    while(dz<-0.5){dz+=1.0;}
	
	    dr2 = dx*dx + dy*dy + dz*dz;
	    if(dr2<cutoff_CuCa*cutoff_CuCa){ACCEPT=0;}
	}

	if(ACCEPT==1)
	{
	    // then accept the atom position
	    Cupos[accepted][0] = trialpos[0];
	    Cupos[accepted][1] = trialpos[1];
	    Cupos[accepted][2] = trialpos[2];
	    accepted++;
	}
	
    }while(ACCEPT==0 || (ACCEPT==1 && accepted<nCu)); 


    // put the phosphorus atoms in

    double cutoff_PO = atof(argv[2])/box;
    double cutoff_PSi = atof(argv[3])/box;
    double cutoff_PNa = atof(argv[3])/box;
    double cutoff_PCa = atof(argv[3])/box;
    double cutoff_PCu = atof(argv[3])/box;
    double cutoff_PP = atof(argv[3])/box;

    double Ppos[nP][3]; // phosphorus atom positions
    accepted=0;
    
    do{
	ACCEPT=1;
	// choose trial position
	trialpos[0] = randone();
	trialpos[1] = randone();
	trialpos[2] = randone();
	
	// check P-O interatomic distances
	for(int j=0; j<nO && ACCEPT==1; j++)
	{
	    dx = Opos[j][0] - trialpos[0];
	    dy = Opos[j][1] - trialpos[1];
	    dz = Opos[j][2] - trialpos[2];
	    
	    // correct for PBCs
	    while(dx>0.5){dx-=1.0;}
	    while(dy>0.5){dy-=1.0;}
	    while(dz>0.5){dz-=1.0;}
	    while(dx<-0.5){dx+=1.0;}
	    while(dy<-0.5){dy+=1.0;}
	    while(dz<-0.5){dz+=1.0;}
	
	    dr2 = dx*dx + dy*dy + dz*dz;
	    if(dr2<cutoff_PO*cutoff_PO){ACCEPT=0;}
	}

	// check P-Si interatomic distances
	for(int j=0; j<nSi && ACCEPT==1; j++)
	{
	    dx = Sipos[j][0] - trialpos[0];
	    dy = Sipos[j][1] - trialpos[1];
	    dz = Sipos[j][2] - trialpos[2];
	    
	    // correct for PBCs
	    while(dx>0.5){dx-=1.0;}
	    while(dy>0.5){dy-=1.0;}
	    while(dz>0.5){dz-=1.0;}
	    while(dx<-0.5){dx+=1.0;}
	    while(dy<-0.5){dy+=1.0;}
	    while(dz<-0.5){dz+=1.0;}
	
	    dr2 = dx*dx + dy*dy + dz*dz;
	    if(dr2<cutoff_PSi*cutoff_PSi){ACCEPT=0;}
	}

	// check P-Na interatomic distances
	for(int j=0; j<nNa && ACCEPT==1; j++)
	{
	    dx = Napos[j][0] - trialpos[0];
	    dy = Napos[j][1] - trialpos[1];
	    dz = Napos[j][2] - trialpos[2];
	    
	    // correct for PBCs
	    while(dx>0.5){dx-=1.0;}
	    while(dy>0.5){dy-=1.0;}
	    while(dz>0.5){dz-=1.0;}
	    while(dx<-0.5){dx+=1.0;}
	    while(dy<-0.5){dy+=1.0;}
	    while(dz<-0.5){dz+=1.0;}
	
	    dr2 = dx*dx + dy*dy + dz*dz;
	    if(dr2<cutoff_PNa*cutoff_PNa){ACCEPT=0;}
	}

	// check P-Ca interatomic distances
	for(int j=0; j<nCa && ACCEPT==1; j++)
	{
	    dx = Capos[j][0] - trialpos[0];
	    dy = Capos[j][1] - trialpos[1];
	    dz = Capos[j][2] - trialpos[2];
	    
	    // correct for PBCs
	    while(dx>0.5){dx-=1.0;}
	    while(dy>0.5){dy-=1.0;}
	    while(dz>0.5){dz-=1.0;}
	    while(dx<-0.5){dx+=1.0;}
	    while(dy<-0.5){dy+=1.0;}
	    while(dz<-0.5){dz+=1.0;}
	
	    dr2 = dx*dx + dy*dy + dz*dz;
	    if(dr2<cutoff_PCa*cutoff_PCa){ACCEPT=0;}
	}

	// check P-P interatomic distances
	for(int j=0; j<accepted && ACCEPT==1; j++)
	{
	    dx = Ppos[j][0] - trialpos[0];
	    dy = Ppos[j][1] - trialpos[1];
	    dz = Ppos[j][2] - trialpos[2];
	    
	    // correct for PBCs
	    while(dx>0.5){dx-=1.0;}
	    while(dy>0.5){dy-=1.0;}
	    while(dz>0.5){dz-=1.0;}
	    while(dx<-0.5){dx+=1.0;}
	    while(dy<-0.5){dy+=1.0;}
	    while(dz<-0.5){dz+=1.0;}
	
	    dr2 = dx*dx + dy*dy + dz*dz;
	    if(dr2<cutoff_PP*cutoff_PP){ACCEPT=0;}
	}

	if(ACCEPT==1)
	{
	    // then accept the atom position
	    Ppos[accepted][0] = trialpos[0];
	    Ppos[accepted][1] = trialpos[1];
	    Ppos[accepted][2] = trialpos[2];
	    accepted++;
	}
	
    }while(ACCEPT==0 || (ACCEPT==1 && accepted<nP)); 

  // put the copper2 atoms in

    double cutoff_Cu2O = atof(argv[2])/box;
    double cutoff_Cu2Si = atof(argv[3])/box;
    double cutoff_Cu2Na = atof(argv[3])/box;
    double cutoff_Cu2Ca = atof(argv[3])/box;
    double cutoff_Cu2Cu = atof(argv[3])/box;
    double cutoff_Cu2P = atof(argv[3])/box;
    double cutoff_Cu2Cu2 = atof(argv[3])/box;

    double Cu2pos[nCu2][3]; // copper 2 atom positions
    accepted=0;
    
    do{
	ACCEPT=1;
	// choose trial position
	trialpos[0] = randone();
	trialpos[1] = randone();
	trialpos[2] = randone();
	
	// check Cu2-O interatomic distances
	for(int j=0; j<nO && ACCEPT==1; j++)
	{
	    dx = Opos[j][0] - trialpos[0];
	    dy = Opos[j][1] - trialpos[1];
	    dz = Opos[j][2] - trialpos[2];
	    
	    // correct for PBCs
	    while(dx>0.5){dx-=1.0;}
	    while(dy>0.5){dy-=1.0;}
	    while(dz>0.5){dz-=1.0;}
	    while(dx<-0.5){dx+=1.0;}
	    while(dy<-0.5){dy+=1.0;}
	    while(dz<-0.5){dz+=1.0;}
	
	    dr2 = dx*dx + dy*dy + dz*dz;
	    if(dr2<cutoff_Cu2O*cutoff_Cu2O){ACCEPT=0;}
	}

	// check Cu2-Si interatomic distances
	for(int j=0; j<nSi && ACCEPT==1; j++)
	{
	    dx = Sipos[j][0] - trialpos[0];
	    dy = Sipos[j][1] - trialpos[1];
	    dz = Sipos[j][2] - trialpos[2];
	    
	    // correct for PBCs
	    while(dx>0.5){dx-=1.0;}
	    while(dy>0.5){dy-=1.0;}
	    while(dz>0.5){dz-=1.0;}
	    while(dx<-0.5){dx+=1.0;}
	    while(dy<-0.5){dy+=1.0;}
	    while(dz<-0.5){dz+=1.0;}
	
	    dr2 = dx*dx + dy*dy + dz*dz;
	    if(dr2<cutoff_Cu2Si*cutoff_Cu2Si){ACCEPT=0;}
	}

	// check Cu2-Na interatomic distances
	for(int j=0; j<nNa && ACCEPT==1; j++)
	{
	    dx = Napos[j][0] - trialpos[0];
	    dy = Napos[j][1] - trialpos[1];
	    dz = Napos[j][2] - trialpos[2];
	    
	    // correct for PBCs
	    while(dx>0.5){dx-=1.0;}
	    while(dy>0.5){dy-=1.0;}
	    while(dz>0.5){dz-=1.0;}
	    while(dx<-0.5){dx+=1.0;}
	    while(dy<-0.5){dy+=1.0;}
	    while(dz<-0.5){dz+=1.0;}
	
	    dr2 = dx*dx + dy*dy + dz*dz;
	    if(dr2<cutoff_Cu2Na*cutoff_Cu2Na){ACCEPT=0;}
	}

	// check Cu2-Ca interatomic distances
	for(int j=0; j<nCa && ACCEPT==1; j++)
	{
	    dx = Capos[j][0] - trialpos[0];
	    dy = Capos[j][1] - trialpos[1];
	    dz = Capos[j][2] - trialpos[2];
	    
	    // correct for PBCs
	    while(dx>0.5){dx-=1.0;}
	    while(dy>0.5){dy-=1.0;}
	    while(dz>0.5){dz-=1.0;}
	    while(dx<-0.5){dx+=1.0;}
	    while(dy<-0.5){dy+=1.0;}
	    while(dz<-0.5){dz+=1.0;}
	
	    dr2 = dx*dx + dy*dy + dz*dz;
	    if(dr2<cutoff_Cu2Ca*cutoff_Cu2Ca){ACCEPT=0;}
	}

       // check Cu2-Cu interatomic distances
	for(int j=0; j<nCu && ACCEPT==1; j++)
	{
	    dx = Cupos[j][0] - trialpos[0];
	    dy = Cupos[j][1] - trialpos[1];
	    dz = Cupos[j][2] - trialpos[2];
	    
	    // correct for PBCs
	    while(dx>0.5){dx-=1.0;}
	    while(dy>0.5){dy-=1.0;}
	    while(dz>0.5){dz-=1.0;}
	    while(dx<-0.5){dx+=1.0;}
	    while(dy<-0.5){dy+=1.0;}
	    while(dz<-0.5){dz+=1.0;}
	
	    dr2 = dx*dx + dy*dy + dz*dz;
	    if(dr2<cutoff_Cu2Cu*cutoff_Cu2Cu){ACCEPT=0;}
	}


       // check Cu2-P interatomic distances
	for(int j=0; j<nP && ACCEPT==1; j++)
	{
	    dx = Ppos[j][0] - trialpos[0];
	    dy = Ppos[j][1] - trialpos[1];
	    dz = Ppos[j][2] - trialpos[2];
	    
	    // correct for PBCs
	    while(dx>0.5){dx-=1.0;}
	    while(dy>0.5){dy-=1.0;}
	    while(dz>0.5){dz-=1.0;}
	    while(dx<-0.5){dx+=1.0;}
	    while(dy<-0.5){dy+=1.0;}
	    while(dz<-0.5){dz+=1.0;}
	
	    dr2 = dx*dx + dy*dy + dz*dz;
	    if(dr2<cutoff_Cu2P*cutoff_Cu2P){ACCEPT=0;}
	}

       // check Cu2-Cu2 interatomic distances
	for(int j=0; j<accepted && ACCEPT==1; j++)
	{
	    dx = Cu2pos[j][0] - trialpos[0];
	    dy = Cu2pos[j][1] - trialpos[1];
	    dz = Cu2pos[j][2] - trialpos[2];
	    
	    // correct for PBCs
	    while(dx>0.5){dx-=1.0;}
	    while(dy>0.5){dy-=1.0;}
	    while(dz>0.5){dz-=1.0;}
	    while(dx<-0.5){dx+=1.0;}
	    while(dy<-0.5){dy+=1.0;}
	    while(dz<-0.5){dz+=1.0;}
	
	    dr2 = dx*dx + dy*dy + dz*dz;
	    if(dr2<cutoff_Cu2Cu2*cutoff_Cu2Cu2){ACCEPT=0;}
	}

	if(ACCEPT==1)
	{
	    // then accept the atom position
	    Cu2pos[accepted][0] = trialpos[0];
	    Cu2pos[accepted][1] = trialpos[1];
	    Cu2pos[accepted][2] = trialpos[2];
	    accepted++;
	}
	
    }while(ACCEPT==0 || (ACCEPT==1 && accepted<nCu2)); 

    // output atom positions in xyz format

    // with the cell parameter at the front
    cout << box << endl;
    if(SHELLMODEL){cout << natoms+nO;}
    else{cout << natoms;}
    cout << endl << 3 << endl;
   
    for (int i=0; i<nO; i++)
    {
      if(SHELLMODEL){
	cout << "Oc " << Opos[i][0]*box << " " << Opos[i][1]*box << " " << Opos[i][2]*box << endl;
	cout << "Os " << Opos[i][0]*box << " " << Opos[i][1]*box << " " << Opos[i][2]*box << endl;
      }
      else
	{
	cout << "O " << Opos[i][0]*box << " " << Opos[i][1]*box << " " << Opos[i][2]*box << endl;
	}
    }
    for (int i=0; i<nSi; i++)
    {
	cout << "Si " << Sipos[i][0]*box << " " << Sipos[i][1]*box << " " << Sipos[i][2]*box << endl;
    }
    for (int i=0; i<nNa; i++)
    {
	cout << "Na " << Napos[i][0]*box << " " << Napos[i][1]*box << " " << Napos[i][2]*box << endl;
    }
    for (int i=0; i<nCa; i++)
    {
	cout << "Ca " << Capos[i][0]*box << " " << Capos[i][1]*box << " " << Capos[i][2]*box << endl;
    }    
    for (int i=0; i<nCu; i++)
    {
	cout << "Cu " << Cupos[i][0]*box << " " << Cupos[i][1]*box << " " << Cupos[i][2]*box << endl;
    }    
    for (int i=0; i<nP; i++)
    {
	cout << "P " << Ppos[i][0]*box << " " << Ppos[i][1]*box << " " << Ppos[i][2]*box << endl;
    }  
    for (int i=0; i<nCu2; i++)
    {
	cout << "Cu2 " << Cu2pos[i][0]*box << " " << Cu2pos[i][1]*box << " " << Cu2pos[i][2]*box << endl;
    }  
  
    // output guidance for field file
    ofstream fout;
    fout.open("notesFIELD");
    fout << nO << " oxygen" << endl;
    fout << nSi << " silicon" << endl;
    fout << nNa << " sodium" << endl;
    fout << nCa << " calcium" << endl;
    fout << nCu << " copper++" << endl;
    fout << nP << " phosphorus" << endl;
    fout << nCu2 << " copper+" << endl;
    fout.close();

    return 0;
}